@@ -2,16 +2,14 @@
 #
 #  Net::Server::PreFork - Net::Server personality
 #
-#  $Id: PreFork.pm,v 1.26 2005/11/15 05:48:04 rhandom Exp $
+#  $Id: PreFork.pm,v 1.34 2007/02/03 05:38:32 rhandom Exp $
 #
-#  Copyright (C) 2001-2005
+#  Copyright (C) 2001-2007
 #
 #    Paul Seamons
 #    paul@seamons.com
 #    http://seamons.com/
 #
-#    Rob Brown bbb@cpan,org
-#
 #  This package may be distributed under the terms of either the
 #  GNU General Public License
 #    or the
@@ -118,19 +116,26 @@ sub loop {
   $prop->{_WRITE} = *_WRITE;
 
   ### get ready for children
-  $prop->{children} = {};
   $prop->{child_select} = IO::Select->new(\*_READ);
+  $prop->{children} = {};
+  if ($ENV{HUP_CHILDREN}) {
+      my %children = map {/^(\w+)$/; $1} split(/\s+/, $ENV{HUP_CHILDREN});
+      $children{$_} = {status => $children{$_}, hup => 1} foreach keys %children;
+      $prop->{children} = \%children;
+  }
 
-  $self->log(3,"Beginning prefork ($prop->{min_servers} processes)\n");
+  my $start = $prop->{min_servers};
+
+  $self->log(3,"Beginning prefork ($start processes)\n");
 
   ### keep track of the status
   $prop->{tally} = {time       => time(),
-                    waiting    => 0,
-                    processing => 0,
-                    dequeue    => 0};
+                    waiting    => scalar(grep {$_->{'status'} eq 'waiting'}    values %{ $prop->{children} }),
+                    processing => scalar(grep {$_->{'status'} eq 'processing'} values %{ $prop->{children} }),
+                    dequeue    => scalar(grep {$_->{'status'} eq 'dequeue'}    values %{ $prop->{children} }),};
 
   ### start up the children
-  $self->run_n_children( $prop->{min_servers} );
+  $self->run_n_children( $start );
 
   ### finish the parent routines
   $self->run_parent;
@@ -150,14 +155,15 @@ sub kill_n_children {
 
   $self->log(3,"Killing \"$n\" children");
 
-  foreach (keys %{ $prop->{children} }){
-    next unless $prop->{children}->{$_}->{status} eq 'waiting';
+  foreach my $child (keys %{ $prop->{children} }){
+    next unless $prop->{children}->{$child}->{status} eq 'waiting';
 
-    $n --;
-    $prop->{tally}->{waiting} --;
+    $n--;
 
     ### try to kill the child
-    kill('HUP',$_) or $self->delete_child( $_ );
+    if (! kill('HUP', $child)) {
+      $self->delete_child($child);
+    }
 
     last if $n <= 0;
   }
@@ -299,6 +305,8 @@ sub run_parent {
                CHLD => sub {
                  while ( defined(my $chld = waitpid(-1, WNOHANG)) ){
                    last unless $chld > 0;
+                   $prop->{tally}->{time} = 0 if $prop->{children}->{$chld}->{hup};
+                   $prop->{tally}->{$prop->{children}->{$chld}->{status}}-- if $prop->{children}->{$chld}->{status};
 		   $self->delete_child( $chld );
                  }
                },
@@ -343,17 +351,18 @@ sub run_parent {
         my ($pid,$status) = ($1,$2);
 
         ### record the status
-        $prop->{children}->{$pid}->{status} = $status;
+        $prop->{children}->{$pid}->{status} = $status
+          if $status ne 'exiting';
+
         if( $status eq 'processing' ){
           $prop->{tally}->{processing} ++;
-          $prop->{tally}->{waiting} --;
           $prop->{last_process} = time();
+
         }elsif( $status eq 'waiting' ){
           $prop->{tally}->{processing} --;
-          $prop->{tally}->{waiting} ++;
+
         }elsif( $status eq 'exiting' ){
-          $prop->{tally}->{processing} --;
-          $self->delete_child( $pid );
+          $self->delete_child($pid);
         }
 
       ### user defined handler
@@ -377,9 +386,9 @@ sub coordinate_children {
   my $prop = $self->{server};
   my $time = time();
 
-  ### re-tally the possible types (only once a minute)
+  ### re-tally the possible types (only twice a minute)
   ### this might not be even necessary but is a nice sanity check
-  if( $time - $prop->{tally}->{time} > 60 ){
+  if( $time - $prop->{tally}->{time} > 30 ){
     my $w = $prop->{tally}->{waiting};
     my $p = $prop->{tally}->{processing};
     $prop->{tally} = {time       => $time,
@@ -434,13 +443,12 @@ sub coordinate_children {
   }
 
   ### periodically make sure children are alive
-  if( $time - $prop->{last_checked_for_dead} > $prop->{check_for_dead} ){
+  if ($time - $prop->{last_checked_for_dead} > $prop->{check_for_dead}) {
     $prop->{last_checked_for_dead} = $time;
-    foreach (keys %{ $prop->{children} }){
+    foreach my $pid (keys %{ $prop->{children} }) {
       ### see if the child can be killed
-      if( ! kill(0,$_) ){
-        $self->delete_child($_);
-        $prop->{tally}->{ $prop->{children}->{status} } --;
+      if (! kill(0, $pid)) {
+        $self->delete_child($pid);
       }
     }
   }
@@ -465,6 +473,24 @@ sub coordinate_children {
 
 }
 
+### delete_child and other modifications contributed by Rob Mueller
+sub delete_child {
+  my $self = shift;
+  my $prop = $self->{server};
+  my $pid = shift;
+
+  # Already gone?
+  return if ! exists $prop->{children}->{$pid};
+
+  my $status = $prop->{children}->{$pid}->{status}
+    || $self->log(2, "No status for $pid when deleting child\n");
+  --$prop->{tally}->{$status} >= 0
+    || $self->log(2, "Tally for $status < 0 deleting pid $pid\n");
+  $prop->{tally}->{time} = 0 if $prop->{children}->{$pid}->{hup};
+
+  $self->SUPER::delete_child($pid);
+}
+
 ### allow for other process to tie in to the parent read
 sub parent_read_hook {}
 
@@ -665,7 +691,7 @@ will be the open socket to the child.
 
 =back
 
-=BUGS
+=head1 BUGS
 
 Tests don't seem to work on Win32.  Any ideas or patches would be welcome.
 
