@@ -2,16 +2,14 @@
 #
 #  Net::Server::PreForkSimple - Net::Server personality
 #
-#  $Id: PreForkSimple.pm,v 1.23 2006/07/08 21:18:35 rhandom Exp $
+#  $Id: PreForkSimple.pm,v 1.26 2007/02/03 08:00:56 rhandom Exp $
 #
-#  Copyright (C) 2001-2005
+#  Copyright (C) 2001-2007
 #
 #    Paul Seamons
 #    paul@seamons.com
 #    http://seamons.com/
 #
-#    Rob Brown bbb@cpan,org
-#
 #  This package may be distributed under the terms of either the
 #  GNU General Public License
 #    or the
@@ -98,6 +96,8 @@ sub post_bind {
       $prop->{lock_file} = POSIX::tmpnam();
       $prop->{lock_file_unlink} = 1;
     }
+    open($prop->{lock_fh}, ">$prop->{lock_file}")
+      || $self->fatal("Couldn't open lock file \"$prop->{lock_file}\"[$!]");
 
   ### set up semaphore
   }elsif( $prop->{serialize} eq 'semaphore' ){
@@ -222,6 +222,8 @@ sub run_child {
 
   $self->child_finish_hook;
 
+  close($prop->{lock_fh}) if $prop->{lock_fh};
+
   $self->log(4,"Child leaving ($prop->{max_requests})");
   exit;
 
@@ -242,13 +244,9 @@ sub accept {
   my $self = shift;
   my $prop = $self->{server};
 
-  local *LOCK;
-
   ### serialize the child accepts
   if( $prop->{serialize} eq 'flock' ){
-    open(LOCK,">$prop->{lock_file}")
-      || $self->fatal("Couldn't open lock file \"$prop->{lock_file}\" [$!]");
-    while (! flock(LOCK, Fcntl::LOCK_EX())) {
+    while (! flock($prop->{lock_fh}, Fcntl::LOCK_EX())) {
       next if $! == EINTR;
       $self->fatal("Couldn't get lock on file \"$prop->{lock_file}\" [$!]");
     }
@@ -268,7 +266,7 @@ sub accept {
 
   ### unblock serialization
   if( $prop->{serialize} eq 'flock' ){
-    flock(LOCK,Fcntl::LOCK_UN());
+    flock($prop->{lock_fh}, Fcntl::LOCK_UN());
 
   }elsif( $prop->{serialize} eq 'semaphore' ){
     $prop->{sem}->op( 0, 1, IPC::SysV::SEM_UNDO() )
@@ -614,7 +612,7 @@ check_for_dequeue setting.
 
 =back
 
-=BUGS
+=head1 BUGS
 
 Tests don't seem to work on Win32.  Any ideas or patches would be welcome.
 
